"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_get_1 = __importDefault(require("lodash.get"));
var lodash_set_1 = __importDefault(require("lodash.set"));
require("reflect-metadata");
var common_1 = require("./common");
function deserialize(json, serializableClass) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    common_1.assertSerializable(serializableClass);
    var propsMetadata = Reflect.getMetadata(common_1.ReflectMetaDataKeys.TsJacksonJsonProperty, serializableClass);
    var resultClass = new (serializableClass.bind.apply(serializableClass, __spreadArray([void 0], args)))();
    var propertiesAfterDeserialize = [];
    for (var _a = 0, _b = Object.entries(propsMetadata); _a < _b.length; _a++) {
        var _c = _b[_a], propName = _c[0], propParams = _c[1];
        var jsonValue = lodash_get_1.default(json, propParams.path);
        propParams.required &&
            common_1.assertRequired({
                json: json,
                propName: propName,
                propValue: jsonValue,
                serializableClass: serializableClass,
                propPath: propParams.path,
            });
        var deserializedValue = propParams.deserialize
            ? propParams.deserialize(jsonValue)
            : deserializeProperty(jsonValue, propParams.type, propParams.elementType);
        propParams.validate &&
            common_1.assertValid({
                propName: propName,
                propValue: deserializedValue,
                validate: propParams.validate,
                serializableClass: serializableClass,
            });
        if (deserializedValue !== undefined) {
            lodash_set_1.default(resultClass, propName, deserializedValue);
        }
        propParams.afterDeserialize &&
            propertiesAfterDeserialize.push({
                propName: propName,
                deserializedValue: deserializedValue,
                afterDeserialize: propParams.afterDeserialize,
            });
    }
    propertiesAfterDeserialize.forEach(function (_a) {
        var propName = _a.propName, deserializedValue = _a.deserializedValue, afterDeserialize = _a.afterDeserialize;
        lodash_set_1.default(resultClass, propName, afterDeserialize(resultClass, deserializedValue));
    });
    return resultClass;
}
exports.default = deserialize;
function deserializeProperty(value, toType, elementType) {
    if (value === undefined || value === null) {
        return value;
    }
    switch (toType === null || toType === void 0 ? void 0 : toType.name) {
        case common_1.Types.Date: {
            return new Date(value);
        }
        case common_1.Types.Array: {
            return value.map(function (item) {
                var isSerializable = common_1.checkSerializable(elementType);
                return isSerializable ? deserialize(item, elementType) : item;
            });
        }
        case common_1.Types.Set: {
            var values = value.map(function (item) {
                var isSerializable = common_1.checkSerializable(elementType);
                return isSerializable ? deserialize(item, elementType) : item;
            });
            return new Set(values);
        }
        case common_1.Types.Boolean: {
            return Boolean(value);
        }
        case common_1.Types.Number: {
            return Number.parseInt(value);
        }
        case common_1.Types.String: {
            return value;
        }
        default: {
            var isSerializable = common_1.checkSerializable(toType);
            return isSerializable
                ? deserialize(value, toType)
                : value;
        }
    }
}
