"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var common_1 = require("./common");
var deserialize_1 = __importDefault(require("./deserialize"));
var JsonProperty_1 = __importDefault(require("./JsonProperty"));
var Serializable_1 = __importDefault(require("./Serializable"));
describe('deserialize', function () {
    test('Class with object field', function () {
        var json = {
            bar: {
                foo: true,
                baz: 'bazName',
            },
        };
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Class.prototype, "bar", void 0);
            Class = __decorate([
                Serializable_1.default()
            ], Class);
            return Class;
        }());
        var expected = new Class();
        expected.bar = json.bar;
        expect(deserialize_1.default(json, Class)).toStrictEqual(expected);
    });
    test('Class with default properties', function () {
        var Class = (function () {
            function Class() {
                this.baz = 'test';
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Class.prototype, "baz", void 0);
            Class = __decorate([
                Serializable_1.default()
            ], Class);
            return Class;
        }());
        expect(deserialize_1.default({}, Class)).toStrictEqual(new Class());
    });
    test('Class with array properties', function () {
        var json = {
            primitiveArray: [1, 2, 3],
            complexArray: [{ foo: 4 }, { baz: true, bar: 'varValue' }],
        };
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Array)
            ], Class.prototype, "primitiveArray", void 0);
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Array)
            ], Class.prototype, "complexArray", void 0);
            Class = __decorate([
                Serializable_1.default()
            ], Class);
            return Class;
        }());
        var expected = new Class();
        expected.primitiveArray = json.primitiveArray;
        expected.complexArray = json.complexArray;
        expect(deserialize_1.default(json, Class)).toEqual(expected);
    });
    test('Class with constructor arguments', function () {
        var json = {
            id: 'idValue',
            bar: {
                foo: true,
                baz: 'bazName',
            },
        };
        var Class = (function () {
            function Class(foo, baz) {
                this.foo = foo;
                this.baz = baz;
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Class.prototype, "id", void 0);
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Class.prototype, "bar", void 0);
            Class = __decorate([
                Serializable_1.default(),
                __metadata("design:paramtypes", [Number, String])
            ], Class);
            return Class;
        }());
        var expected = new Class(4, 'baz');
        expected.id = json.id;
        expected.bar = json.bar;
        expect(deserialize_1.default(json, Class, 4, 'baz')).toStrictEqual(expected);
    });
    test('Class with required field', function () {
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default({ required: true }),
                __metadata("design:type", String)
            ], Class.prototype, "id", void 0);
            Class = __decorate([
                Serializable_1.default()
            ], Class);
            return Class;
        }());
        expect(function () { return deserialize_1.default({}, Class); }).toThrow(new common_1.RequiredPropertyError({
            serializableClass: Class,
            json: {},
            propName: 'id',
            propPath: 'id',
        }));
    });
    test('Class with one to one relation', function () {
        var Dog = (function () {
            function Dog() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Dog.prototype, "name", void 0);
            Dog = __decorate([
                Serializable_1.default()
            ], Dog);
            return Dog;
        }());
        var Owner = (function () {
            function Owner() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Owner.prototype, "name", void 0);
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Dog)
            ], Owner.prototype, "dog", void 0);
            Owner = __decorate([
                Serializable_1.default()
            ], Owner);
            return Owner;
        }());
        var json = {
            name: 'Shaggy',
            dog: {
                name: 'Scooby Doo',
            },
        };
        var dog = new Dog();
        dog.name = json.dog.name;
        var expected = new Owner();
        expected.dog = dog;
        expected.name = json.name;
        expect(deserialize_1.default(json, Owner)).toStrictEqual(expected);
    });
    test('Class with one to many relation', function () {
        var Dog = (function () {
            function Dog() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Dog.prototype, "name", void 0);
            Dog = __decorate([
                Serializable_1.default()
            ], Dog);
            return Dog;
        }());
        var Owner = (function () {
            function Owner() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Owner.prototype, "name", void 0);
            __decorate([
                JsonProperty_1.default({ elementType: Dog }),
                __metadata("design:type", Array)
            ], Owner.prototype, "dogs", void 0);
            Owner = __decorate([
                Serializable_1.default()
            ], Owner);
            return Owner;
        }());
        var json = {
            name: 'Shaggy',
            dogs: [
                {
                    name: 'Scooby Doo',
                },
                {
                    name: 'Scrappy Doo',
                },
            ],
        };
        var scoobyDoo = new Dog();
        scoobyDoo.name = json.dogs[0].name;
        var scrappyDoo = new Dog();
        scrappyDoo.name = json.dogs[1].name;
        var expected = new Owner();
        expected.dogs = [scoobyDoo, scrappyDoo];
        expected.name = json.name;
        expect(deserialize_1.default(json, Owner)).toStrictEqual(expected);
    });
    test('"Set" property type', function () {
        var Dog = (function () {
            function Dog() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Dog.prototype, "name", void 0);
            Dog = __decorate([
                Serializable_1.default()
            ], Dog);
            return Dog;
        }());
        var Owner = (function () {
            function Owner() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Owner.prototype, "name", void 0);
            __decorate([
                JsonProperty_1.default({ elementType: Dog }),
                __metadata("design:type", Set)
            ], Owner.prototype, "dogs", void 0);
            Owner = __decorate([
                Serializable_1.default()
            ], Owner);
            return Owner;
        }());
        var json = {
            name: 'Shaggy',
            dogs: [
                {
                    name: 'Scooby Doo',
                },
                {
                    name: 'Scrappy Doo',
                },
            ],
        };
        var scoobyDoo = new Dog();
        scoobyDoo.name = json.dogs[0].name;
        var scrappyDoo = new Dog();
        scrappyDoo.name = json.dogs[1].name;
        var expected = new Owner();
        expected.dogs = new Set([scoobyDoo, scrappyDoo]);
        expected.name = json.name;
        expect(deserialize_1.default(json, Owner)).toStrictEqual(expected);
    });
    describe('Inheritance', function () {
        var Profile = (function () {
            function Profile() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Profile.prototype, "id", void 0);
            Profile = __decorate([
                Serializable_1.default()
            ], Profile);
            return Profile;
        }());
        var Manager = (function (_super) {
            __extends(Manager, _super);
            function Manager() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Manager.prototype, "role", void 0);
            return Manager;
        }(Profile));
        it('should assume valid', function () {
            expect(function () { return deserialize_1.default({}, Manager); }).not.toThrow(new common_1.SerializableError(Manager));
        });
        it("should take into account inherited properties marked with @" + JsonProperty_1.default.name, function () {
            var json = {
                id: 'idValue',
                role: 'assistant-manager',
            };
            var expected = new Manager();
            expected.id = json.id;
            expected.role = json.role;
            expect(deserialize_1.default(json, Manager)).toStrictEqual(expected);
        });
    });
    describe('validate', function () {
        var validate = function (property) { return lodash_1.inRange(property, 18, 99); };
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default({
                    validate: validate,
                }),
                __metadata("design:type", Number)
            ], Class.prototype, "age", void 0);
            Class = __decorate([
                Serializable_1.default()
            ], Class);
            return Class;
        }());
        test('should throw an Error if property failed validate check', function () {
            var invalidData = {
                age: 14,
            };
            expect(function () { return deserialize_1.default(invalidData, Class); }).toThrow(new common_1.ValidatePropertyError({
                propName: 'age',
                propValue: 14,
                validate: validate,
                serializableClass: Class,
            }));
        });
        test('should not throw an Error if property pass validate check', function () {
            var validData = {
                age: 18,
            };
            expect(function () { return deserialize_1.default(validData, Class); }).not.toThrow(common_1.ValidatePropertyError);
        });
    });
    it("Should throw an error if class is not annotated with " + Serializable_1.default.name, function () {
        var Class = (function () {
            function Class() {
            }
            return Class;
        }());
        expect(function () { return deserialize_1.default({}, Class); }).toThrow(new common_1.SerializableError(Class));
    });
    test('Custom deserialize param', function () {
        var json = {
            full_name: {
                name: 'John',
                surname: 'Jackson',
            },
        };
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default({
                    path: 'full_name',
                    deserialize: function (jsonValue) {
                        return jsonValue.name + ' ' + jsonValue.surname;
                    },
                }),
                __metadata("design:type", String)
            ], Class.prototype, "fullName", void 0);
            Class = __decorate([
                Serializable_1.default()
            ], Class);
            return Class;
        }());
        var expected = new Class();
        expected.fullName = json.full_name.name + ' ' + json.full_name.surname;
        expect(deserialize_1.default(json, Class)).toStrictEqual(expected);
    });
    describe('afterDeserialize', function () {
        test('Date', function () {
            var json = {
                expires_in: 3600,
            };
            var Class = (function () {
                function Class() {
                }
                __decorate([
                    JsonProperty_1.default({
                        path: 'expires_in',
                        afterDeserialize: function (_, propertyValue) {
                            return new Date(Date.now() + propertyValue.getTime());
                        },
                    }),
                    __metadata("design:type", Date)
                ], Class.prototype, "expiresAt", void 0);
                Class = __decorate([
                    Serializable_1.default()
                ], Class);
                return Class;
            }());
            var expected = new Class();
            var expiresAt = new Date(Date.now() + json.expires_in);
            expected.expiresAt = expiresAt;
            var deserializedValue = deserialize_1.default(json, Class);
            expect(deserializedValue.expiresAt.getTime()).toBeGreaterThanOrEqual(expiresAt.getTime());
            expect(deserializedValue.expiresAt.getTime()).toBeLessThan(expiresAt.getTime() + 100);
        });
        test('String', function () {
            var Class = (function () {
                function Class() {
                }
                __decorate([
                    JsonProperty_1.default(),
                    __metadata("design:type", String)
                ], Class.prototype, "name", void 0);
                __decorate([
                    JsonProperty_1.default({
                        path: 'surname',
                        afterDeserialize: function (deserializedInstance, propertyValue) {
                            return deserializedInstance.name + ' ' + propertyValue;
                        },
                    }),
                    __metadata("design:type", String)
                ], Class.prototype, "fullName", void 0);
                Class = __decorate([
                    Serializable_1.default()
                ], Class);
                return Class;
            }());
            var json = {
                name: 'John',
                surname: 'Jones',
            };
            var expected = new Class();
            expected.name = json.name;
            expected.fullName = json.name + ' ' + json.surname;
            expect(deserialize_1.default(json, Class)).toEqual(expected);
        });
    });
});
