"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_set_1 = __importDefault(require("lodash.set"));
var common_1 = require("./common");
function serialize(instance) {
    common_1.assertSerializable(instance.constructor);
    var propsMetadata = Reflect.getMetadata(common_1.ReflectMetaDataKeys.TsJacksonJsonProperty, instance.constructor);
    var json = {};
    for (var _i = 0, _a = Object.entries(propsMetadata); _i < _a.length; _i++) {
        var _b = _a[_i], propName = _b[0], propParams = _b[1];
        var serializedProperty = propParams.serialize
            ? propParams.serialize(instance[propName])
            : serializeProperty(instance[propName], propParams.type, propParams.elementType);
        lodash_set_1.default(json, propParams.path, serializedProperty);
    }
    return json;
}
exports.default = serialize;
function serializeProperty(value, type, elementType) {
    if (value === undefined) {
        return value;
    }
    switch (type === null || type === void 0 ? void 0 : type.name) {
        case common_1.Types.Array: {
            return value.map(function (item) {
                var isSerializable = common_1.checkSerializable(elementType);
                return isSerializable ? serialize(item) : item;
            });
        }
        case common_1.Types.Set: {
            return Array.from(value.values()).map(function (item) {
                var isSerializable = common_1.checkSerializable(elementType);
                return isSerializable ? serialize(item) : item;
            });
        }
        default: {
            var isSerializable = common_1.checkSerializable(type);
            return isSerializable
                ? serialize(value)
                : value;
        }
    }
}
