"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("../index");
var common_1 = require("./common");
var JsonProperty_1 = __importDefault(require("./JsonProperty"));
describe('serialize', function () {
    it("should throw an error if class is not annotated with " + index_1.Serializable.name, function () {
        var Class = (function () {
            function Class() {
            }
            return Class;
        }());
        expect(function () { return index_1.serialize(new Class()); }).toThrow(new common_1.SerializableError(Class));
    });
    test('simple paths', function () {
        var json = {
            foo: 'test',
            baz: 5,
            bars: ['hello', 'star'],
        };
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", String)
            ], Class.prototype, "foo", void 0);
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Number)
            ], Class.prototype, "baz", void 0);
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Array)
            ], Class.prototype, "bars", void 0);
            Class = __decorate([
                index_1.Serializable()
            ], Class);
            return Class;
        }());
        var testClass = new Class();
        testClass.foo = json.foo;
        testClass.baz = json.baz;
        testClass.bars = json.bars;
        expect(index_1.serialize(testClass)).toEqual(testClass);
    });
    test('nested paths', function () {
        var json = {
            nested: {
                foo: 'test',
                bars: ['hello', 'star'],
            },
        };
        var Class = (function () {
            function Class() {
            }
            __decorate([
                JsonProperty_1.default('nested.foo'),
                __metadata("design:type", String)
            ], Class.prototype, "foo", void 0);
            __decorate([
                JsonProperty_1.default('nested.bars[0]'),
                __metadata("design:type", String)
            ], Class.prototype, "bar1", void 0);
            __decorate([
                JsonProperty_1.default('nested.bars[1]'),
                __metadata("design:type", String)
            ], Class.prototype, "bar2", void 0);
            Class = __decorate([
                index_1.Serializable()
            ], Class);
            return Class;
        }());
        var testClass = new Class();
        testClass.foo = json.nested.foo;
        testClass.bar1 = json.nested.bars[0];
        testClass.bar2 = json.nested.bars[1];
        expect(index_1.serialize(testClass)).toEqual(json);
    });
    test('inheritance', function () {
        var Parent = (function () {
            function Parent() {
                this.foo = 'fooValue';
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Parent.prototype, "foo", void 0);
            Parent = __decorate([
                index_1.Serializable()
            ], Parent);
            return Parent;
        }());
        var Child = (function (_super) {
            __extends(Child, _super);
            function Child() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.baz = 'bazValue';
                return _this;
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Child.prototype, "baz", void 0);
            return Child;
        }(Parent));
        expect(index_1.serialize(new Child())).toEqual({
            foo: 'fooValue',
            baz: 'bazValue',
        });
    });
    test('custom serialize param', function () {
        var Class = (function () {
            function Class() {
                this.fullName = 'Jack Johns';
            }
            __decorate([
                JsonProperty_1.default({
                    serialize: function (fullName) {
                        var _a = fullName.split(' '), name = _a[0], surname = _a[1];
                        return {
                            name: name,
                            surname: surname,
                        };
                    },
                }),
                __metadata("design:type", Object)
            ], Class.prototype, "fullName", void 0);
            Class = __decorate([
                index_1.Serializable()
            ], Class);
            return Class;
        }());
        expect(index_1.serialize(new Class())).toEqual({
            fullName: {
                name: 'Jack',
                surname: 'Johns',
            },
        });
    });
    test('one to one relation', function () {
        var json = {
            name: 'Shaggy',
            dog: {
                name: 'Scooby Doo',
            },
        };
        var Dog = (function () {
            function Dog() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Dog.prototype, "name", void 0);
            Dog = __decorate([
                index_1.Serializable()
            ], Dog);
            return Dog;
        }());
        var Owner = (function () {
            function Owner() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Owner.prototype, "name", void 0);
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Dog)
            ], Owner.prototype, "dog", void 0);
            Owner = __decorate([
                index_1.Serializable()
            ], Owner);
            return Owner;
        }());
        var dog = new Dog();
        dog.name = json.dog.name;
        var owner = new Owner();
        owner.name = json.name;
        owner.dog = dog;
        expect(index_1.serialize(owner)).toStrictEqual(json);
    });
    test('one to many relation', function () {
        var Dog = (function () {
            function Dog() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Dog.prototype, "name", void 0);
            Dog = __decorate([
                index_1.Serializable()
            ], Dog);
            return Dog;
        }());
        var Owner = (function () {
            function Owner() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Owner.prototype, "name", void 0);
            __decorate([
                JsonProperty_1.default({ elementType: Dog }),
                __metadata("design:type", Array)
            ], Owner.prototype, "dogs", void 0);
            Owner = __decorate([
                index_1.Serializable()
            ], Owner);
            return Owner;
        }());
        var json = {
            name: 'Shaggy',
            dogs: [
                {
                    name: 'Scooby Doo',
                },
                {
                    name: 'Scrappy Doo',
                },
            ],
        };
        var scoobyDoo = new Dog();
        scoobyDoo.name = json.dogs[0].name;
        var scrappyDoo = new Dog();
        scrappyDoo.name = json.dogs[1].name;
        var owner = new Owner();
        owner.name = json.name;
        owner.dogs = [scoobyDoo, scrappyDoo];
        expect(index_1.serialize(owner)).toStrictEqual(json);
    });
    test('"Set" property type', function () {
        var Dog = (function () {
            function Dog() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Dog.prototype, "name", void 0);
            Dog = __decorate([
                index_1.Serializable()
            ], Dog);
            return Dog;
        }());
        var Owner = (function () {
            function Owner() {
            }
            __decorate([
                JsonProperty_1.default(),
                __metadata("design:type", Object)
            ], Owner.prototype, "name", void 0);
            __decorate([
                JsonProperty_1.default({ elementType: Dog }),
                __metadata("design:type", Set)
            ], Owner.prototype, "dogs", void 0);
            Owner = __decorate([
                index_1.Serializable()
            ], Owner);
            return Owner;
        }());
        var json = {
            name: 'Shaggy',
            dogs: [
                {
                    name: 'Scooby Doo',
                },
                {
                    name: 'Scrappy Doo',
                },
            ],
        };
        var scoobyDoo = new Dog();
        scoobyDoo.name = json.dogs[0].name;
        var scrappyDoo = new Dog();
        scrappyDoo.name = json.dogs[1].name;
        var owner = new Owner();
        owner.name = json.name;
        owner.dogs = new Set([scoobyDoo, scrappyDoo]);
        expect(index_1.serialize(owner)).toStrictEqual(json);
    });
});
